import * as i0 from '@angular/core';
import { InjectionToken, forwardRef, Directive, Host, Inject, Input, NgModule } from '@angular/core';
import * as i1 from '@angular/forms';
import { FormGroup, FormArray } from '@angular/forms';

const NG_INJECTABLE_CONTROL = new InjectionToken('ng-injectable-control');
function injectableControlProvider(type) {
    return {
        provide: NG_INJECTABLE_CONTROL,
        useExisting: forwardRef(() => type),
    };
}

/**
 * Work around for ExpressionChangeAfterCheckedError from
 * https://stackoverflow.com/questions/45661010/dynamic-nested-reactive-form-expressionchangedafterithasbeencheckederror
 */
const resolvedPromise = (() => Promise.resolve(null))();
class InjectControlDirective {
    constructor(parent, host) {
        this.parent = parent;
        this.host = host;
    }
    set disabled(isDisabled) {
        const control = this.host.control;
        if (isDisabled && control.enabled) {
            control.disable();
        }
        else if (!isDisabled && control.disabled) {
            control.enable();
        }
        this.isDisabled = isDisabled;
    }
    ngOnChanges(changes) {
        const change = changes.controlName;
        if (change) {
            if (!change.firstChange) {
                this._tearDownControl(change.previousValue);
            }
            this._setUpControl();
        }
    }
    ngOnDestroy() {
        this._tearDownControl(this.controlName);
    }
    _setUpControl() {
        const parent = this.parent.control;
        const control = resolveControl(parent, this.controlName);
        if (!control) {
            throw new Error(`No control ${this.controlName} in control container`);
        }
        this.originalControl = control;
        resolvedPromise.then(() => {
            copyControlState(this.host, control, this.isDisabled);
            replaceInParent(parent, this.controlName, this.host.control);
        });
    }
    _tearDownControl(name) {
        const parent = this.parent.control;
        resolvedPromise.then(() => {
            if (this.originalControl) {
                replaceInParent(parent, name, this.originalControl);
            }
        });
    }
}
InjectControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: InjectControlDirective, deps: [{ token: i1.ControlContainer }, { token: NG_INJECTABLE_CONTROL, host: true }], target: i0.ɵɵFactoryTarget.Directive });
InjectControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.1.1", type: InjectControlDirective, selector: "[injectControl]", inputs: { controlName: ["injectControl", "controlName"], disabled: "disabled" }, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: InjectControlDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[injectControl]',
                }]
        }], ctorParameters: function () { return [{ type: i1.ControlContainer }, { type: undefined, decorators: [{
                    type: Host
                }, {
                    type: Inject,
                    args: [NG_INJECTABLE_CONTROL]
                }] }]; }, propDecorators: { controlName: [{
                type: Input,
                args: ['injectControl']
            }], disabled: [{
                type: Input
            }] } });
function copyControlState(host, control, isDisabled) {
    if (control.value) {
        host.control.patchValue(control.value);
    }
    if (isDisabled === undefined && control.enabled) {
        host.control.enable();
    }
    if (isDisabled === undefined && control.disabled) {
        host.control.disable();
    }
    if (control.validator) {
        host.control.setValidators(control.validator);
        host.control.updateValueAndValidity({
            onlySelf: true,
            emitEvent: false,
        });
    }
    if (control.asyncValidator) {
        host.control.setAsyncValidators(control.asyncValidator);
        host.control.updateValueAndValidity({
            onlySelf: true,
            emitEvent: false,
        });
    }
}
function resolveControl(parent, name) {
    if (parent instanceof FormGroup) {
        if (typeof name !== 'string') {
            throw new Error(`name must be a string`);
        }
        return parent.get(name);
    }
    else if (parent instanceof FormArray) {
        if (typeof name !== 'number') {
            throw new Error(`name must be a number`);
        }
        return parent.at(name);
    }
    throw new Error(`Unsupported control container type ${parent?.constructor.name}`);
}
function replaceInParent(parent, name, control) {
    if (parent instanceof FormGroup) {
        parent.setControl(name, control);
    }
    else {
        parent.setControl(name, control);
    }
    parent.updateValueAndValidity({ emitEvent: false });
}

class InjectControlModule {
}
InjectControlModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: InjectControlModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
InjectControlModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: InjectControlModule, declarations: [InjectControlDirective], exports: [InjectControlDirective] });
InjectControlModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: InjectControlModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: InjectControlModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [InjectControlDirective],
                    exports: [InjectControlDirective],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { InjectControlDirective, InjectControlModule, NG_INJECTABLE_CONTROL, injectableControlProvider };
//# sourceMappingURL=ng-inject-control.mjs.map
